#!/usr/bin/env python3
import os
import pathlib

libRootPath = os.path.join(os.path.dirname(os.path.realpath(__file__)),"..")
docDir = os.path.join(libRootPath,"Doc")
inputDir = os.path.join("input")
outputDir = os.path.join("output")
featuresDir = os.path.join("data","shader_code","features")
utilsDir = os.path.join("data","shader_code","utils")
templatesDir = os.path.join("data","Templates")



#!/usr/bin/env python3
import os
import re
import sys
import pkg_resources
from LandSeed.LibPaths import libRootPath

# class used to get all inforamtions about a feaeture or a util
class ShaderFragmentInfo:
    # initialize the object by reading the file path
    def __init__(self, featOrUtil, path):
        self.cat = featOrUtil # 2 values : "feature" or "util"
        if pkg_resources.resource_exists("LandSeed", path):
            self.path = pkg_resources.resource_filename("LandSeed",path)
        else:
            self.path = path
        self.tag = ""
        self.funcName = ""
        self.signature = ""
        self.name = ""
        self.short = ""
        self.spec = ""
        self.tagPresence = False
        self.dependencies = []
        self.functionCode = ""
        self.beginLine = 0
        self.extractInfo()
    # extract all info from the file define by path
    def extractInfo(self):
        if os.path.isfile(self.getPath()):
            fragmentFile = open(self.getPath(), "r")
            fragmentContent = fragmentFile.readlines()
            fragmentFile.close()
            for line in range(len(fragmentContent)):
                if "@TAG" in fragmentContent[line] and fragmentContent[line][fragmentContent[line].find("@TAG")+4] != "\n":
                    p = re.compile("@TAG (.*)")
                    tag = p.search(fragmentContent[line]).group(1)
                    if " " in tag:
                        print("File badly formated : tag should be in one single word in "+self.getPath().replace(libRootPath, ""))
                        sys.exit(1)
                    self.setTag(tag)
                elif "@FUNCTION_NAME" in fragmentContent[line] and fragmentContent[line][fragmentContent[line].find("@FUNCTION_NAME")+14] != "\n":
                    p = re.compile("@FUNCTION_NAME (.*)")
                    functionName = p.search(fragmentContent[line]).group(1)
                    self.setFunctionName(functionName)
                elif "@SIGNATURE" in fragmentContent[line] and fragmentContent[line][fragmentContent[line].find("@SIGNATURE")+10] != "\n":
                    p = re.compile("@SIGNATURE (.*)")
                    signature = p.search(fragmentContent[line]).group(1)
                    self.setSignature(signature)
                elif "@NAME" in fragmentContent[line] and fragmentContent[line][fragmentContent[line].find("@NAME")+5] != "\n":
                    p = re.compile("@NAME (.*)")
                    fullName = p.search(fragmentContent[line]).group(1)
                    self.setName(fullName)
                elif "@SPEC" in fragmentContent[line]:
                    line += 1
                    spec = fragmentContent[line].replace("// ", "")+"\n"
                    line += 1
                    while not "//}" in fragmentContent[line].replace(" ", ""):
                        spec += "- "+fragmentContent[line].replace("// ", "")+"\n"
                        line += 1
                    self.setSpec(spec)
                elif "@SHORT" in fragmentContent[line] and fragmentContent[line][fragmentContent[line].find("@SHORT")+6] != "\n":
                    p = re.compile("@SHORT (.*)")
                    self.setShortDesc(p.search(fragmentContent[line]).group(1))
                if "//END" in fragmentContent[line].replace(" ","").replace("-", ""):
                    if self.isHeaderComplete():
                        break
                    else:
                        print("Header badly formated in "+self.getPath().replace(libRootPath, ""))
                        del self
                        sys.exit(1)
            while (line < len(fragmentContent)) and (not self.tagPresence):
                if "@"+self.getTag() in fragmentContent[line]:
                    self.tagPresence = True
                line += 1
            if not self.tagPresence:
                print("Missing starting tag : @"+self.tag+" before function implementation start in "+self.getPath().replace(libRootPath, ""))
                del self
                sys.exit(1)

            while "@INCLUDE" in fragmentContent[line]:
                p = re.compile("@INCLUDE (.*)") # TODO check case where #INCLUDE
                result = p.search(fragmentContent[line]).group(1)
                self.dependencies.append(result)
                line += 1
            self.beginLine = line
            while line < len(fragmentContent) and not "@END" in fragmentContent[line]:
                self.functionCode += fragmentContent[line]
                line += 1

            if line >= len(fragmentContent):
                print("Missing @END tag in "+self.getPath().replace(libRootPath, ""))
                del self
                sys.exit(1)
            line +=1
        else:
            print("Wrong path to file given : "+self.getPath().replace(libRootPath, ""))
            del self
            sys.exit(2)
    def getPath(self):
        return self.path
    def setTag(self, str):
        self.tag = str
    def setFunctionName(self, str):
        self.funcName = str
    def setSignature(self, str):
        self.signature = str
    def setName(self, str):
        self.name = str
    def setSpec(self, str):
        self.spec = str
    def getTag(self):
        return self.tag
    def getFunctionName(self):
        return self.funcName
    def getSignature(self):
        return self.signature
    def getName(self):
        return self.name
    def getSpec(self):
        return self.spec
    def getCat(self):
        return self.cat
    def getRelativePath(self):
        return self.path.replace(libRootPath, "")
    def getAbsolutePath(self):
        return self.path
    def getDependencies(self):
        return self.dependencies
    def getBeginLine(self):
        return self.beginLine
    def getFunctionCode(self):
        return self.functionCode
    def getShortDesc(self):
        return self.short
    def setShortDesc(self, str):
        self.short = str
    # check if the object contains all the informations it should
    def isHeaderComplete(self):
        if not (self.tag == "" or self.funcName == "" or self.signature == "" or self.name == "" or self.spec == "" or self.short == ""):
            return True
        return False
    # display informations about the fragment. Used to debug
    def displayInfo(self):
        print("-----------------------------------------")
        print("Category: "+self.cat)
        print("Tag: "+self.tag)
        print("Function Name: "+self.funcName)
        print("Signature: "+self.signature)
        print("Name: "+self.name)
        print("Spec: "+self.spec)
        print("Tag presence: "+str(self.tagPresence))
        print("-----------------------------------------")

    # convert the object to markdown format for the documentation page
    def toMD(self):
        str = ""
        str += "# "+self.name+"\n\n"
        str += "**Tag**: "+self.tag+"\n\n"
        str += "**Category**: "+self.cat+"\n\n"
        str += "**Function Name**: "+self.funcName+"\n\n"
        str += "**Signature**: "+self.signature+"\n\n"
        str += "**Spec**: "+self.spec+"\n\n"
        str += "**Path**: "+self.path.replace(libRootPath, "")+"\n\n"
        return str



#!/usr/bin/env python3
import os
import re
import pkg_resources
from pydoc import locate
import sys
# try: # Pas beau mais gère le fait qu'on puisse aussi appeler le script depuis de dossier src/LibUtils/
#     from src.LibUtils.LibPaths import libRootPath
# except Exception as e:
#     from LibPaths import libRootPath
from LandSeed.LibPaths import libRootPath
# param spec in the params array for each indices:
# 0 : TAG
# 1 : type
# 2 : file where it needs to be replaced
# 3 : Default value
# 4 : Description for the documentation

class TemplateInfo:
    # initialize the object by reading the file path, from the line beginLine (for the case where there is more than one fragment in the same file)
    def __init__(self, path):
        if not path[-1:]=="/":
            path += "/"
        self.name = ""
        self.tag = ""
        self.fileToFill = ""
        self.pathToFileToFill = ""
        self.lineDirective = None
        self.desc = ""
        self.params = []
        if pkg_resources.resource_exists("LandSeed", path):
            self.path = pkg_resources.resource_filename("LandSeed",path)
        else:
            self.path = path
        self.content = []
        if not os.path.isdir(self.getPath()): # Check if the template dir exist
            print("The template path given should be a directory containing all template files.\nWrong file path given : "+path)
            sys.exit(2)
        self.extractInfo()
        self.extractContent()
        if not self.isComplete():
            print("Please, fulfill the template.config file following the template described in README")
            sys.exit(1)
        self.checkParamType()

    def extractContent(self):
        if not os.path.exists(self.getPathToFileToFill()):
            print("Wrong path given to the file to fulfill in the template.\nTemplate : "+self.path)
            sys.exit(2)
        shaderFile = open(self.getPathToFileToFill(), "r")
        self.content = shaderFile.readlines()
        shaderFile.close()

    def extractInfo(self):
        if not os.path.exists(os.path.join(self.path,"template.config")):
            print("template.config missing in the template at : "+self.path)
            sys.exit(2)
        configFile = open(os.path.join(self.path,"template.config"), "r")
        configFileContent = configFile.readlines()
        configFile.close()
        for line in configFileContent:
            if "@NAME" in line and len(line.split(" ")) > 1:
                p = re.compile("@NAME (.*)")
                res = p.search(line).group(1)
                self.setName(res)
            elif "@TAG" in line and len(line.split(" ")) == 2:
                self.setTag((line.replace("\n", "")).split(" ")[1])
            elif "@GEN_FILE" in line and len(line.split(" ")) == 2:
                self.setFileToFillName((line.replace("\n","")).split(" ")[1])
                self.setPathToFileToFill(os.path.join(self.path, self.getFileTofillName()))
            elif "@LINE_DIRECTIVE_ON" in line and len(line.split(" ")) == 2:
                res = line.split(" ")[1]
                if "true" in res.lower():
                    self.setLineDirective(True)
                elif "false" in res.lower():
                    self.setLineDirective(False)
                else:
                    print("@LINE_DIRECTIVE_ON should be a boolean in config file at "+ self.path)
                    sys.exit(1)
            elif "@DESC" in line and len(line.split(" ")) > 2:
                p = re.compile("@DESC { (.*) }")
                if p.search(line) != None:
                    res = p.search(line).group(1)
                    self.setDesc(res)
                else:
                    print("@DESC tag badly formatted : Description should be between brackets like that { ... }\nIn template : "+self.path)
                    sys.exit(1)
            elif "@PARAM" in line and not "@PARAM_DESC" in line:
                if not len((line.replace("\n", "")).split(" ")) == 5:
                    print("Param line in template.config file badly written.\nPlease follow the way to write it in the README.\nIn template : "+self.path)
                    sys.exit(2)
                line_split = (line.replace("\n", "")).split(" ")
                del line_split[0]
                if line_split[0] == "" or line_split[1] == "" or line_split[2] == "" or line_split[3] == "":
                    print("Param line in template.config file badly written.\nPlease follow the way to write it in the README.\nIn template : "+self.path)
                    sys.exit(2)
                line_split[1] = line_split[1].lower()
                if line_split[2] == "GEN_FILE":
                    line_split[2] = self.getPathToFileToFill()
                else:
                    line_split[2] = os.path.join(self.getPath(), line_split[2])
                p = re.compile("DEFAULT=(.*)")
                line_split[3] = p.search(line_split[3]).group(1)
                self.params.append(line_split)
            elif "@PARAM_DESC" in line:
                try:
                    p = re.compile("@PARAM_DESC (.*) {")
                    param_name = p.search(line).group(1)
                    p = re.compile("@PARAM_DESC "+param_name+" { (.*) }")
                    param_desc = p.search(line).group(1)
                except Exception as e:
                    print("PARAM_DESC badly written in template.config in "+self.path)
                    sys.exit(1)
                for param in self.params:
                    if param[0] == param_name:
                        param.append(param_desc)

    def checkParamType(self):
        for p in self.getParams():
            try: # try a convertion to detecte some type error (not efficient for all possibilities : 123 to bool will be convert as True and no error will be raised)
                locate(self.getParamType(p).lower())(self.getParamDefaultValue(p))
            except Exception as e:
                print("Wrong type of paramter given in template.config for "+self.getParamTag(p)+".\nThe type should be "+self.getParamType(p))
                sys.exit(1)

    def isComplete(self): # TODO fulfill
        return self.lineDirective != None and self.name != "" and self.desc != "" and self.tag != "" and self.fileToFill != ""

    # use the display function to debug
    def displayInfo(self):
        print("-------------------------------------------")
        print("Name : "+self.getName())
        print("Tag : "+self.getTag())
        print("Line directive on : "+str(self.getLineDirective()))
        print("Desc : "+self.getDesc())
        print("FileToFill : "+self.getFileTofillName())
        print("Path : "+self.getPath())
        print("pathToFileToFill : "+self.getPathToFileToFill())
        print("PARAMS")
        for p in self.params:
            print(repr(p))
        print("-------------------------------------------")
        print("Content : ")
        print(repr(self.getContent()))

    def toMD(self):
        res = ""
        res += "# "+self.getName()+"\n\n"
        res += "**Tag**: "+self.getTag()+"\n\n"
        res += "**Description**: "+self.getDesc()+"\n\n"
        res += "**Path**: "+self.getPath().replace(libRootPath, "")+"\n\n"
        res += "**Parameters**:\n\n"
        res += "| Tag | Type | Default Value | File | Description |\n"
        res += "|-|-|-|-|-|\n"
        for p in self.getParams():
            res += "| "+self.getParamTag(p)+" | "+self.getParamType(p)+" | "+str(self.getParamDefaultValue(p))+" | "+self.getParamFile(p)+" | "+self.getParamDescription(p)+" |\n"
        return res

    def getParams(self):
        return self.params
    def getNbOfParam(self):
        return len(self.params)
    def paramToString(self, param):
        print(param[0]+" : "+param[1]+", default value : "+str(param[3])+"; "+param[4])
    def getParamTag(self, param):
        return param[0]
    def getParamType(self, param):
        return param[1]
    def getParamFileFullPath(self, param):
        return param[2]
    def getParamFile(self, param):
        return param[2].replace(self.path, "")
    def getParamDefaultValue(self, param):
        return param[3]
    def getParamDescription(self, param):
        return param[4]

    def setLineDirective(self, on):
        self.lineDirective = on
    def getLineDirective(self):
        return self.lineDirective
    def setName(self, n):
        self.name = n
    def getName(self):
        return self.name
    def setTag(self, n):
        self.tag = n
    def getTag(self):
        return self.tag
    def setDesc(self, d):
        self.desc = d
    def getDesc(self):
        return self.desc
    def getContent(self):
        return self.content
    def setFileToFillName(self, str):
        self.fileToFill = str
    def getFileTofillName(self):
        return self.fileToFill
    def setPath(self, str):
        self.path = str
    def getPath(self):
        return self.path
    def setPathToFileToFill(self, str):
        self.pathToFileToFill = str
    def getPathToFileToFill(self):
        return self.pathToFileToFill



#!/usr/bin/env python3

# Error exception meaning :
#   FileBadlyWrittenException : keyword missing in a file or badly written
#   PathErrorException : path error
#   DependencyException : dependency not recognize
#   WrongParameterException : script parameter error
#   TooOldPythonVersionException : Python version too old

class Error(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(message)

class FileBadlyWrittenException(Error):
    def __init__(self, file):
        super().__init__(message)

class PathErrorException(Error):
    def __init__(self, message):
        super().__init__(message)

class DependencyException(Error):
    def __init__(self, message):
        super().__init__(message)

class WrongParameterException(Error):
    def __init__(self, message):
        super().__init__(message)

class TooOldPythonVersionException(Error):
    def __init__(self, message):
        super().__init__(message)



import setuptools
import os

def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for dir in directories:
            paths.append(os.path.join(path.replace('LandSeed/', ''),dir, "*"))
    return paths

extra_files = package_files('LandSeed/data')

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="LandSeed",
    version="1.0.12",
    author="Bastien Zigmann",
    author_email="bastien@zigmann.org",
    description="Procedural terrain shader generator",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Sauww/LandSeed",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires='>=3.0',
    install_requires=[
        'numpy>=1.18.1',
        'glfw>=1.11.1',
        'PyOpenGL>=3.1.5'
    ],
    entry_points = {
        "console_scripts": [
            "LandSeed=LandSeed.LandSeed:generate",
            "LandSeed_newinput=LandSeed.LandSeed:newInputFile"
        ]
    },
    include_package_data=True,
    package_data={'':extra_files+["input/demo*.frag"],},
)

# TODO :
#  - check classifier



#!/usr/bin/env python3
import sys
if sys.version_info.major < 3: # python version should be 3+
    print("Python version to old, please upgrade your python to 3 or more.")
    raise PythonVersionException("Python version too old")
if sys.version_info.minor >= 4:
    import importlib
else:
    import imp
import os
from pydoc import locate
import re
import queue
import shutil
import pkg_resources
from LandSeed.LibPaths import libRootPath, inputDir, outputDir, featuresDir, utilsDir, templatesDir
from LandSeed.TemplateInfoClass import TemplateInfo
from LandSeed.ShaderFragmentInfoClass import ShaderFragmentInfo

# Return code meaning :
#   0 : everything's ok
#   1 : keyword missing in a file or badly written
#   2 : path error
#   3 : dependency not recognize
#   4 : script parameter error
#   5 : index file error
#   6 : Python version too old

sys.path.append(libRootPath)

includedFeatures = [] # to register which feature we already added
includedDependencies = [] # to register which dependencies we already added

def writeLineDirective(line, file):
    if template.getLineDirective():
        outputFile.write("#line "+str(line)+" \""+file+"\"\n")

# Return index of the line where keyword has been found. Create an error and leave script if the keyword is not found
# fileContent : where to search
# keyword : what to search
# filename : name of the file with his path inside the lib for error printing
def skipUntil(fileContent, keyword, fileName):
    line = 0
    while (line < len(fileContent)) and (not keyword in fileContent[line]): # skipe until keyword is found or EOF is reached
        line += 1
    if line >= len(fileContent):
        error("Keyword " + keyword + " not found in " + fileName, 1)
    else:
        return line

# Exit the script properly with errMessage printed and the code errCode
def error(errorMessage, errCode):
    print(errorMessage)
    if outputFile != None:
        outputFile.close()
    if os.path.exists(outputPath):
        shutil.rmtree(outputPath)
    sys.exit(errCode)

# include a dependency a the dependencies of this dependency recursively
def includeDependency(dependencyName):
    global includedFeatures
    global includedDependencies
    dependency = None
    # if this dependency hasn't been added yet
    if not dependencyName.lower() in includedDependencies and not dependencyName.lower() in includedFeatures:

        for feat in pkg_resources.resource_listdir("LandSeed", featuresDir):
            if feat.replace(".frag", "").upper() == dependencyName.upper():
                dependencyPath = os.path.join(featuresDir, feat)
                dependency = ShaderFragmentInfo("feature", dependencyPath)
                includedFeatures.append(dependencyName.lower())
                break
        if dependency == None:
            for util in pkg_resources.resource_listdir("LandSeed", utilsDir):
                if util.replace(".frag", "").upper() == dependencyName.upper():
                    dependencyPath = os.path.join(utilsDir, util)
                    dependency = ShaderFragmentInfo("utils", dependencyPath)
                    includedDependencies.append(dependencyName.lower())
                    break
        if dependency == None:
            error("Dependency not recognize : "+dependencyName, 3)

        for dep in dependency.getDependencies():
            includeDependency(dep)

        print("Including "+dependency.getCat()+" "+dependencyName+"...")
        writeLineDirective(dependency.getBeginLine()+1, dependencyPath.replace(libRootPath, ""))
        outputFile.write(dependency.getFunctionCode())


# start including input.
# Run two times through the input file's line
# the first time, detect every features used and include them
# the second time, assuming all dependencies have been included, copy the terrainMap function.
def includeTerrainMap(input, outputFile):
    # include what is necessary
    line = 0
    line = skipUntil(input, "@FEATURES", inputPath) # get the line number where is the keyword FEATURES

    # add each features detected and her dependencies
    # for each not commented lines, if there is a feature in it, call includeFeature
    while (line < len(input)) and not "@END" in input[line] :
         #if the line is not commented
        if not "//" in input[line].replace(" ", "")[0:2]:
            # check every features
            for featureFile in pkg_resources.resource_listdir("LandSeed", featuresDir) :
                feature = featureFile.replace(".frag","")
                # if the feature is in the line, include it
                if feature in input[line] and (input[line].replace(" ", "")[input[line].replace(" ", "").find(feature)+len(feature)]=="("):
                    includeDependency(feature)
            for utilFile in pkg_resources.resource_listdir("LandSeed", utilsDir) : # check util after feature
                util = utilFile.replace(".frag","")
                # if the util is in the line, include it
                if util in input[line] and (input[line].replace(" ", "")[input[line].replace(" ", "").find(util)+len(util)]=="("):
                    includeDependency(util)
        line +=1

    # if we have reached the end of the file without finding any @END tag
    if line >= len(input):
        error("@END tag missing in input/input.frag file", 2)
    writeLineDirective(1, inputPath.replace(libRootPath, ""))
    # finally copy the terrainMap function after adding all dependencies
    for line in input :
        outputFile.write(line)
    outputFile.seek(0) # used in order to readlines again to count the number of lines
    writeLineDirective(len(outputFile.readlines())+2, outputFilePath.replace(libRootPath, ""))
    outputFile.write("\n")


# copy the template into the output shader and detect where includes have to be done
def copyAndComplete(input):
    global template
    global outputFilePath
    templateName = None
    firstLines = 0
    while firstLines < len(input) and (templateName == None):
        if "@TEMPLATE" in input[firstLines] and input[firstLines][input[firstLines].find("@TEMPLATE")+9] == " ":
            inputFileTemplateLine = input[firstLines].replace("// ", "").replace("//", "").replace("\n", "").split(" ")
            templateName = inputFileTemplateLine[1]
            # if not templateName in os.listdir(templatesDir):
            if not pkg_resources.resource_exists("LandSeed", os.path.join(templatesDir, templateName)):
                error("Unknown template name : "+templateName, 3)
        firstLines += 1
    if firstLines >= len(input):
        if templateName == None:
            error("Missing template declaration in input file.", 1)
    print("Template : "+templateName)

    templateDirPath=os.path.join(templatesDir, templateName)

    template = TemplateInfo(templateDirPath)
    # getting template content
    emptyShaderContent = template.getContent()
    shutil.copytree(template.getPath(), outputPath,ignore=shutil.ignore_patterns('template.config'))

    outputFilePath=os.path.join(outputPath, template.getFileTofillName())

    # if output file exists, remove it, then open it
    if os.path.exists(outputFilePath):
        os.remove(outputFilePath)
    global outputFile

    if len(inputFileTemplateLine) < 2 or len(inputFileTemplateLine) > 2 + template.getNbOfParam(): # check if the input file got enough param (this means no param or all params)
        print("Please declare correctly template in input file.\nYou can declare it alone or with some of the parameters.")
        for param in template.getParams():
            template.paramToString(param)
        sys.exit(1)

    # init tag list in order to mark those who are defined by the user then use the non-marked defaults values
    paramUsed = {}
    for p in template.getParams():
        paramUsed[template.getParamTag(p)]= False

    for paramDeclared in range(2,len(inputFileTemplateLine)): # run through each parameters declared by the user in the input file
        if inputFileTemplateLine[paramDeclared]: # if there is a parameter
            try: # try to parse it getting to two parts of the "TAG=VALUE" declaration
                currentParamTagRE = re.compile("(.*)=")
                currentParamTag = currentParamTagRE.search(inputFileTemplateLine[paramDeclared]).group(1)
                currentParamGivenValueRE = re.compile("=(.*)")
                currentParamGivenValue = currentParamGivenValueRE.search(inputFileTemplateLine[paramDeclared]).group(1)
            except Exception as e:
                print("Error while parsing input file.")
                sys.exit(1)
            if currentParamTag == "" or currentParamGivenValue == "":
                print("One of the parameter you entered has no value or no tag")
                sys.exit(1)
            if currentParamTag in paramUsed.keys():
                paramUsed[currentParamTag] = True
            else:
                print("Template param "+currentParamTag+" used in the input is not recognize.\nPlease use the following parameters correctly.")
                for param in template.getParams():
                    template.paramToString(param)
                sys.exit(1)
            for templateParam in template.getParams(): # search for the parameter in all the parameters defined in the template
                if template.getParamTag(templateParam) == currentParamTag :
                    try: # try a convertion to detecte some type error (not efficient for all possibilities : 123 to bool will be convert as True and no error will be raised)
                        locate(template.getParamType(templateParam).lower())(currentParamGivenValue)
                    except Exception as e:
                        print("Wrong type of paramter given for "+currentParamTag+".\nThe type should be "+template.getParamType(templateParam))
                        sys.exit(1)
                    if template.getParamFile(templateParam) == template.getFileTofillName(): # if the param should be put in GEN_FILE
                        for line in range(len(emptyShaderContent)):
                            if "@"+template.getParamTag(templateParam) in emptyShaderContent[line]:
                                emptyShaderContent[line] = emptyShaderContent[line].replace("@"+currentParamTag, currentParamGivenValue)
                    else: # if it's a custom file
                        print("---------------DEBUG-------------")
                        print("outputPath : "+outputPath)
                        print("ParamFile : "+template.getParamFile(p))
                        print(os.path.join(outputPath, template.getParamFile(p)))
                        print("---------------DEBUG-------------")
                        fileToFulfillPath = os.path.join(outputPath, template.getParamFile(p))
                        fin = open(fileToFulfillPath, "rt")
                        data = fin.read()
                        data = data.replace("@"+currentParamTag, currentParamGivenValue)
                        fin.close()
                        fin = open(fileToFulfillPath, "wt")
                        fin.write(data)
                        fin.close()

    # fill missing params with with defaults values
    for templateParam in template.getParams():
        if paramUsed[template.getParamTag(templateParam)] == False:
            print(template.getParamTag(templateParam)+" default value is taken")
            if template.getParamFile(templateParam) == template.getFileTofillName(): # if the param should be put in GEN_FILE
                for line in range(len(emptyShaderContent)):
                    if "@"+template.getParamTag(templateParam) in emptyShaderContent[line]:
                        emptyShaderContent[line] = emptyShaderContent[line].replace("@"+template.getParamTag(templateParam), template.getParamDefaultValue(templateParam))
            else: # if it's a custom file
                fileToFulfillPath = os.path.join(outputPath, template.getParamFile(p))
                fin = open(fileToFulfillPath, "rt")
                data = fin.read()
                data = data.replace("@"+template.getParamTag(templateParam), template.getParamDefaultValue(templateParam))
                fin.close()
                fin = open(fileToFulfillPath, "wt")
                fin.write(data)
                fin.close()

    outputFile = open(outputFilePath, "w+")
    outputFile.write("// @FROM "+inputPath.replace(libRootPath,"")+"\n")
    outputFile.write("// @TO "+os.path.dirname(outputFilePath.replace(libRootPath, ""))+"\n")
    # run through every lines of the template, seeking for the @TERRAIN_MAP tag.
    # if it's not present, copy the current line then go on the next one
    # if it's on the line, include the input and all the dependencies
    # for line in range(line, len(emptyShaderContent)):
    for line in range(len(emptyShaderContent)):
        if not "@TERRAIN_MAP" in emptyShaderContent[line]:
            if line == 1:
                writeLineDirective(3, outputFilePath.replace(libRootPath, ""))
            outputFile.write(emptyShaderContent[line])
        else :
            includeTerrainMap(input, outputFile)
    outputFile.close()

# in order to make the genreation from another file
def generate(inputP=os.path.join(inputDir,"demo.frag"), output=os.path.join(os.getcwd(),outputDir), forceOverwrite=False):

    global outputPath # path to the output dir
    global outputFile # output file (the one where @TERRAIN_MAP is)
    global inputPath # path to the input file
    global includedFeatures
    global includedDependencies

    includedFeatures = [] # to register which feature we already added
    includedDependencies = [] # to register which dependencies we already added

    if len(sys.argv) == 3:
        inputPath, outputPath = os.path.join(sys.argv[1]),os.path.join(sys.argv[2])
    elif len(sys.argv) == 1:
        print("Default parameters are taken.")
        inputPath = inputP
        outputPath = output
    else:
        print("Parameter error.")
        print("Syntax : ")
        print("LandSeed.py [inputPath] [outputPath]")
        print("or")
        print("LandSeed.py")
        sys.exit(4)
    # inputPath = os.path.join(libRootPath,input)
    print("Input file : "+inputPath)
    # outputPath = os.path.join(libRootPath,output)
    print("Output file : "+outputPath)
    # checking that input exist and that it's a file
    if (not os.path.exists(inputPath) or not os.path.isfile(inputPath)) and not pkg_resources.resource_exists("LandSeed", inputPath) :
        print("Please enter a valid or existing input file.")
        sys.exit(4)

    # opening and getting input file content
    if pkg_resources.resource_exists("LandSeed", inputPath):
        inputFile = open(pkg_resources.resource_filename("LandSeed", inputPath), "r")
    else:
        inputFile = open(inputPath, "r")
    inputFileContent = inputFile.readlines()
    inputFile.close()

    outputFile = None

    if os.path.exists(outputPath) and forceOverwrite == False:
        choice = input(outputPath + " already exists, are you sure you want to overwrite it (y/n)? ")
        if choice.upper() == "Y":
            shutil.rmtree(outputPath)
        else:
            print("Aborting..")
            sys.exit(-1)
    elif os.path.exists(outputPath) and forceOverwrite == True:
        shutil.rmtree(outputPath)
    # fulfill template with input file and dependencies
    copyAndComplete(inputFileContent)

    return 0

def newInputFile():
    print("Generating new input file")
    shutil.copyfile(pkg_resources.resource_filename("LandSeed", os.path.join(inputDir,"demo.frag")), "input.frag")
    return 0

# Todo : make this usable
if __name__=="__main__":
        # user can enter a personnal input file or use the default one
        # no parameters -> default input and output
        if len(sys.argv)==1:
            print("Default input file is taken : "+os.path.join(inputDir, "input.frag"))
            print("Default output directory is taken : "+outputDir)
            generate(os.path.join(inputDir,"input.frag"), outputDir)
        # parameters given
        elif len(sys.argv)==3:
            print("Input file : "+sys.argv[1])
            print("Output file : "+sys.argv[2])
            generate(sys.argv[1], sys.argv[2])
        # wrong number of parameters
        else:
            print("Parameter error.")
            print("Syntax : ")
            print("LandSeed.py [inputPath] [outputPath]")
            print("or")
            print("LandSeed.py")
            sys.exit(4)

        # exit
        sys.exit(0)
